/*
 * Copyright (c) Institute for System Programming of the Russian Academy of Sciences
 * All Rights Reserved
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
 * 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
 * http://www.ispras.ru
 *
 * The present file was automatically generated on the basis of formal specifications.
 * It is distributed under the same terms and conditions as the derived specifications.
 *
 * N.B. PLEASE DO NOT MODIFY THIS FILE.
 */

package ru.ispras.microtesk.model.x86;

import java.math.BigInteger;
import ru.ispras.microtesk.model.api.data.TypeId;
import ru.ispras.microtesk.model.api.data.Type;
import ru.ispras.microtesk.model.api.memory.Memory;
import ru.ispras.microtesk.model.api.memory.Label;

public final class Shared {
  private Shared() {}
  public static final String PC = "program_counter";

  public static final Type BIT = Type.def("BIT", Type.CARD(1));
  public static final Type BYTE = Type.def("BYTE", Type.CARD(8));
  public static final Type SBYTE = Type.def("SBYTE", Type.INT(8));
  public static final Type SIZE = Type.def("SIZE", Type.CARD(3));
  public static final Type ADDR = Type.def("ADDR", Type.CARD(20));
  public static final Type WORD = Type.def("WORD", Type.CARD(16));
  public static final Type SWORD = Type.def("SWORD", Type.INT(16));
  public static final Type DWORD = Type.def("DWORD", Type.CARD(32));
  public static final Type INDEX = Type.def("INDEX", Type.CARD(3));

  public static final Memory program_counter = Memory.def(Memory.Kind.VAR, "program_counter", ADDR, 0x1);
  public static final Memory GPR = Memory.def(Memory.Kind.REG, "GPR", WORD, 0x8);
  public static final Memory AX = Memory.def(Memory.Kind.REG, "AX", WORD, 0x1, GPR.access(0));
  public static final Memory AL = Memory.def(Memory.Kind.REG, "AL", BYTE, 0x1, AX.access().bitField(0, 7));
  public static final Memory AH = Memory.def(Memory.Kind.REG, "AH", BYTE, 0x1, AX.access().bitField(8, 15));
  public static final Memory CX = Memory.def(Memory.Kind.REG, "CX", WORD, 0x1, GPR.access(1));
  public static final Memory CL = Memory.def(Memory.Kind.REG, "CL", BYTE, 0x1, CX.access().bitField(0, 7));
  public static final Memory CH = Memory.def(Memory.Kind.REG, "CH", BYTE, 0x1, CX.access().bitField(8, 15));
  public static final Memory DX = Memory.def(Memory.Kind.REG, "DX", WORD, 0x1, GPR.access(2));
  public static final Memory DL = Memory.def(Memory.Kind.REG, "DL", BYTE, 0x1, DX.access().bitField(0, 7));
  public static final Memory DH = Memory.def(Memory.Kind.REG, "DH", BYTE, 0x1, DX.access().bitField(8, 15));
  public static final Memory BX = Memory.def(Memory.Kind.REG, "BX", WORD, 0x1, GPR.access(3));
  public static final Memory BL = Memory.def(Memory.Kind.REG, "BL", BYTE, 0x1, BX.access().bitField(0, 7));
  public static final Memory BH = Memory.def(Memory.Kind.REG, "BH", BYTE, 0x1, BX.access().bitField(8, 15));
  public static final Memory SP = Memory.def(Memory.Kind.REG, "SP", WORD, 0x1, GPR.access(4));
  public static final Memory BP = Memory.def(Memory.Kind.REG, "BP", WORD, 0x1, GPR.access(5));
  public static final Memory SI = Memory.def(Memory.Kind.REG, "SI", WORD, 0x1, GPR.access(6));
  public static final Memory DI = Memory.def(Memory.Kind.REG, "DI", WORD, 0x1, GPR.access(7));
  public static final Memory RSEG = Memory.def(Memory.Kind.REG, "RSEG", WORD, 0x4);
  public static final Memory ES = Memory.def(Memory.Kind.REG, "ES", WORD, 0x1, RSEG.access(0));
  public static final Memory CS = Memory.def(Memory.Kind.REG, "CS", WORD, 0x1, RSEG.access(1));
  public static final Memory SS = Memory.def(Memory.Kind.REG, "SS", WORD, 0x1, RSEG.access(2));
  public static final Memory DS = Memory.def(Memory.Kind.REG, "DS", WORD, 0x1, RSEG.access(3));
  public static final Memory IP = Memory.def(Memory.Kind.REG, "IP", WORD, 0x1);
  public static final Memory FLAGS = Memory.def(Memory.Kind.REG, "FLAGS", WORD, 0x1);
  public static final Memory CF = Memory.def(Memory.Kind.VAR, "CF", BIT, 0x1, FLAGS.access().bitField(0, 0));
  public static final Memory PF = Memory.def(Memory.Kind.VAR, "PF", BIT, 0x1, FLAGS.access().bitField(2, 2));
  public static final Memory AF = Memory.def(Memory.Kind.VAR, "AF", BIT, 0x1, FLAGS.access().bitField(4, 4));
  public static final Memory ZF = Memory.def(Memory.Kind.VAR, "ZF", BIT, 0x1, FLAGS.access().bitField(6, 6));
  public static final Memory SF = Memory.def(Memory.Kind.VAR, "SF", BIT, 0x1, FLAGS.access().bitField(7, 7));
  public static final Memory TF = Memory.def(Memory.Kind.VAR, "TF", BIT, 0x1, FLAGS.access().bitField(8, 8));
  public static final Memory IF = Memory.def(Memory.Kind.VAR, "IF", BIT, 0x1, FLAGS.access().bitField(9, 9));
  public static final Memory DF = Memory.def(Memory.Kind.VAR, "DF", BIT, 0x1, FLAGS.access().bitField(10, 10));
  public static final Memory OF = Memory.def(Memory.Kind.VAR, "OF", BIT, 0x1, FLAGS.access().bitField(11, 11));
  public static final Memory M = Memory.def(Memory.Kind.MEM, "M", BYTE, 0x100000);
  public static final Memory rm = Memory.def(Memory.Kind.VAR, "rm", Type.CARD(3), 0x1);
  public static final Memory mod = Memory.def(Memory.Kind.VAR, "mod", Type.CARD(2), 0x1);
  public static final Memory instr_size = Memory.def(Memory.Kind.VAR, "instr_size", SIZE, 0x1);
  public static final Memory base = Memory.def(Memory.Kind.VAR, "base", ADDR, 0x1);
  public static final Memory tmpdw_op1 = Memory.def(Memory.Kind.VAR, "tmpdw_op1", DWORD, 0x1);
  public static final Memory tmpdw_op2 = Memory.def(Memory.Kind.VAR, "tmpdw_op2", DWORD, 0x1);
  public static final Memory tmpb_op1 = Memory.def(Memory.Kind.VAR, "tmpb_op1", BYTE, 0x1);
  public static final Memory tmpb_op2 = Memory.def(Memory.Kind.VAR, "tmpb_op2", BYTE, 0x1);
  public static final Memory tmpb_res = Memory.def(Memory.Kind.VAR, "tmpb_res", Type.CARD(9), 0x1);
  public static final Memory tmpw_op1 = Memory.def(Memory.Kind.VAR, "tmpw_op1", WORD, 0x1);
  public static final Memory tmpw_op2 = Memory.def(Memory.Kind.VAR, "tmpw_op2", WORD, 0x1);
  public static final Memory tmpw_res = Memory.def(Memory.Kind.VAR, "tmpw_res", Type.CARD(17), 0x1);

  public static final Memory[] __REGISTERS = {GPR, AX, AL, AH, CX, CL, CH, DX, DL, DH, BX, BL, BH, SP, BP, SI, DI, RSEG, ES, CS, SS, DS, IP, FLAGS};
  public static final Memory[] __MEMORY = {M};
  public static final Memory[] __VARIABLES = {program_counter, CF, PF, AF, ZF, SF, TF, IF, DF, OF, rm, mod, instr_size, base, tmpdw_op1, tmpdw_op2, tmpb_op1, tmpb_op2, tmpb_res, tmpw_op1, tmpw_op2, tmpw_res};
  public static final Label[] __LABELS = {new Label("PC", program_counter, 0)};
} 